// JavaScript object interface for WebFigure

/*
 * CONFIDENTIAL AND CONTAINING PROPRIETARY TRADE SECRETS
 * Copyright 2007 The MathWorks, Inc.
 * The source code contained in this listing contains proprietary and
 * confidential trade secrets of The MathWorks, Inc.  The use, modification,
 * or development of derivative work based on the code or ideas obtained
 * from the code is prohibited without the express written permission of The
 * MathWorks, Inc. The disclosure of this code to any party not authorized
 * by The MathWorks, Inc. is strictly forbidden.
 * CONFIDENTIAL AND CONTAINING PROPRIETARY TRADE SECRETS
 */

if (!mathworks.webfigures) {
    mathworks.webfigures = {};
}

mathworks.include("/mathworks/webfigures/WebFigureLoader.js");    
mathworks.include("/mathworks/util/ajax.js");
mathworks.include("/mathworks/quadtree/quadtree.js");

mathworks.webfigures.RenderMode = {
    IMAGE : {
        createRenderedView : function(figure,viewParams) {
            var element = document.createElement('img');
            element.src = figure.getImageURL(viewParams);
            return element;
        }
    },
    TILEMAP : {
        createRenderedView : function(figure,viewParams) {
            var tileSize = 128;
            if ('tileSize' in viewParams) {
                tileSize = viewParams.tileSize;
            }
            var tileWidth = tileSize;
            if ('tileWidth' in viewParams) {
                tileWidth = viewParams.tileWidth;
            }
            var tileHeight = tileSize;
            if ('tileHeight' in viewParams) {
                tileHeight = viewParams.tileHeight;
            }
            
            var numRows = 3;
            var numCols = 3;
            if ('height' in viewParams) {
                numRows = parseInt((parseInt(viewParams.height, 10) + tileHeight - 1) / tileHeight, 10);
            }
            if ('width' in viewParams) {
                numCols = parseInt((parseInt(viewParams.width, 10) + tileWidth - 1) / tileWidth, 10);
            }

            var tileMap = mathworks.tilemap.create(numRows,numCols, 
                    mathworks.UrlFactoryImpl.getResourceUrl(mathworks.webfigures.imageRoot + 'transparent_pixel.gif')); 
            
            if ('width' in viewParams) {
                tileMap.style.width = viewParams.width;
            }
            if ('height' in viewParams) {
                tileMap.style.height = viewParams.height;
            }

            tileMap.disableLoading = false;
            
            tileMap.figure = figure;

            tileMap.reveal = function(clipRect) {
                if (0 === clipRect.width || 0 === clipRect.height) {
                    return;
                }
                var rowTop = 0;
                var rowBottom;
                for (var r = 0; r < this.numRows; ++r) {
                    rowBottom = rowTop + this.getTile(r,0).offsetHeight;
                    var colLeft = 0;
                    var colRight;
                    for (var c = 0; c < this.numCols; ++c) {
                        var imgElement = this.getTile(r,c);
                        if (null === imgElement) {
                            continue;
                        }
                        colRight = colLeft + imgElement.offsetWidth;
                        if(colLeft <= (clipRect.left+clipRect.width) && clipRect.left <= colRight && 
                           rowTop <= (clipRect.top+clipRect.height) && clipRect.top <= rowBottom) {
                            if (!tileMap.disableLoading) {
                                var params = {};
                                params.width = this.offsetWidth;
                                params.height = this.offsetHeight;
                                params.cropLeft = colLeft;
                                params.cropRight = this.offsetWidth - colRight;
                                params.cropTop = rowTop;
                                params.cropBottom = this.offsetHeight - rowBottom;
                                var url = this.figure.getImageURL(params);
                                if (url != imgElement.src) {
                                    imgElement.src = url;
                                }
                            }
                            imgElement.style.visibility = 'inherit';
                        } else {
                            imgElement.style.visibility = 'hidden';
                        }
                        colLeft = colRight;
                    }
                    rowTop = rowBottom;
                }
            };

            return tileMap;
        }
    },
    QUADTREE :  {
        createRenderedView : function(figure,viewParams) {
            var tileSize = 256;
            if ('tileSize' in viewParams) {
                tileSize = viewParams.tileSize;
            }
            var tileWidth = tileSize;
            if ('tileWidth' in viewParams) {
                tileWidth = viewParams.tileWidth;
            }
            var tileHeight = tileSize;
            if ('tileHeight' in viewParams) {
                tileHeight = viewParams.tileHeight;
            }
            
            return mathworks.quadtree.create(figure, tileWidth*tileHeight, viewParams.scheduler, viewParams);
        }
    }
};

mathworks.webfigures.WebFigure = function (urlFactory) {
    this.UrlFactory = urlFactory;
};

mathworks.webfigures.WebFigure.prototype = {        
    getImageURL: function(viewParams) {
        //create and populate a full parameter object to send to the urlfactory
        //add in all of the viewParam fields
        var params = {};
        for (var p in viewParams) {
            if (viewParams.hasOwnProperty(p)) {
                params[p] = viewParams[p];
            }
        }

        return this.UrlFactory.getRenderUrl(params);
    },

    render: function(viewParams, mode) {
        if (undefined === mode) {
            mode = mathworks.webfigures.RenderMode.IMAGE;
        }
        return mode.createRenderedView(this, viewParams);
    },
    
    /**
     * Retreive properties of the WebFigure and return them asynchronously.
     *
     * The structure of the callback object is:
     *
     * {
     *     onResult: function(properties) {...}
     * }
     *
     * The structure of the properties object returned is:
     *
     * {
     *     backgroundColor: '#xxxxxx',
     *     width: number,
     *     height: number,
     *     azimuth: number,
     *     elevation: number
     * } 
     *
     */
    getProperties: function(callback) {
        var url = this.UrlFactory.getPropertiesUrl();
        mathworks.util.Ajax.makeJSONRequest(url, function(result,obj) {obj.onResult(result);}, callback);
    }
};




