// GraphLab Project: http://graphlab.sharif.edu
// Copyright (C) 2008 Mathematical Science Department of Sharif University of Technology
// Distributed under the terms of the GNU General Public License (GPL): http://www.gnu.org/licenses/
package graphlab.graph.old;

import graphlab.graph.graph.EdgeModel;
import graphlab.graph.graph.GraphPoint;
import graphlab.graph.graph.VertexModel;
import graphlab.platform.lang.ArrayX;
import graphlab.platform.lang.FromStringProvider;
import graphlab.platform.preferences.GraphPreferences;
import graphlab.platform.preferences.UserDefinedEligiblity;
import graphlab.platform.preferences.lastsettings.StorableOnExit;
import graphlab.platform.preferences.lastsettings.UserModifiableProperty;

import java.awt.*;
import java.util.HashMap;
import java.util.Vector;


public class ArrowHandler implements StorableOnExit, UserDefinedEligiblity, FromStringProvider<Arrow> {
    //I thinked and decided to choose the base arrows to be Polygons. i think it will be enough for the most needs, but it can't handle more complex needs, like arrows that are images.

    @UserModifiableProperty(displayName = "Default Arrwo Size for Edges")
    public static Integer arrowSize = 10;


    /**
     * draws the arrow of the edge on the graphics
     *
     * @param zoomFactor the zoom factor of Graph Model
     */
    public static void paint(Graphics _g, EdgeModel e, double zoomFactor) {
//        new ArrowHandler();
        Arrow arrow = e.getArrow();
        Graphics2D g = (Graphics2D) _g;

        double angle = e.getAngle();

        VertexModel v2 = e.target;
        GraphPoint v2ShapeSize = v2.getSize();
        int w = (int) v2ShapeSize.getX();
        int h = (int) v2ShapeSize.getY();
        double t = Math.atan2(w * Math.sin(angle), h * Math.cos(angle));
        GraphPoint loc = e.target.getLocation();
        int x2 = (int) ((zoomFactor * loc.x) + ((w / 2) * Math.cos(t)));
        int y2 = (int) ((zoomFactor * loc.y) + ((h / 2) * Math.sin(t)));

        g.translate(x2 + 1, y2 + 1);
        g.rotate(angle + Math.PI);
        arrow.paintArrow(g, arrowSize, arrowSize);
        g.rotate(-angle - Math.PI);
        g.translate(-x2 - 1, -y2 - 1);
    }

    public static PolygonArrow defaultArrow;

    static {
        //create and add default arrows
        int xPoints[] = {0, -15, -15};
        int yPoints[] = {0, -15 / 2, 15 / 2};
        defaultArrow = new PolygonArrow(new Polygon(xPoints, yPoints, 3), "Default");
        PolygonArrow ar1 = new PolygonArrow(new Polygon(new int[]{0, -15, -10, -15}, new int[]{0, -7, 0, 7}, 4), "Narrow");
        PolygonArrow ar2 = new PolygonArrow(new Polygon(new int[]{0, -15, -5, -15}, new int[]{0, -7, 0, 7}, 4), "Very Narrow");
        PolygonArrow ar3 = new PolygonArrow(new Polygon(new int[]{0, -15, -30, -15}, new int[]{0, -7, 0, 7}, 4), "Box");
        knownArrows = new Vector<Arrow>();
        registerArrow(defaultArrow);
        registerArrow(ar1);
        registerArrow(ar2);
        registerArrow(ar3);

    }

    public static Vector<Arrow> knownArrows;

    public static void registerArrow(Arrow arrow) {
        knownArrows.add(arrow);
    }

    public GraphPreferences GraphPrefFactory() {
        return new GraphPreferences(this.getClass().getSimpleName(), this, "Graph Drawings");
    }

    public HashMap<Object, ArrayX> defineEligibleValuesForSettings(HashMap<Object, ArrayX> objectValues) {
        ArrayX t = new ArrayX(new Integer(3));
        t.addValidValue(new Integer(10));
        t.addValidValue(new Integer(20));
        t.addValidValue(new Integer(30));
//        t.addValidValue(15);

        try {
            objectValues.put(this.getClass().getField("arrowSize"), t);
        } catch (NoSuchFieldException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        return objectValues;
    }

    public Arrow fromString(String toString) {
        for (Arrow s : knownArrows) {
            if (s.getName().equals(toString)) {
                return s;
            }
        }
        return null;
    }
}