// GraphLab Project: http://graphlab.sharif.edu
// Copyright (C) 2008 Mathematical Science Department of Sharif University of Technology
// Distributed under the terms of the GNU Lesser General Public License (LGPL): http://www.gnu.org/licenses/

package graphlab.library.algorithms.graphdecomposition;

import graphlab.library.BaseEdge;
import graphlab.library.BaseGraph;
import graphlab.library.BaseVertex;
import graphlab.library.algorithms.Algorithm;
import graphlab.library.algorithms.AutomatedAlgorithm;
import graphlab.library.algorithms.util.EventUtils;
import graphlab.library.event.GraphRequest;
import graphlab.library.util.Pair;

import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

/**
 * This Method find the biconnected components of a
 * graph.
 * Which are the maximal subgraphs without any cut vertices.
 *
 * @author Soroush Sabet
 */
public class BiconnectedComponents
        <VertexType extends BaseVertex, EdgeType extends BaseEdge<VertexType>>
        extends Algorithm implements AutomatedAlgorithm {
    //PreWorkPostWorkHandler handler = new PreWorkPostWorkHandler<VertexType>();
    Integer[] DFS_N;
    Integer[] High;
    int[] Parent;
    Vector<Pair<Vector<VertexType>, Vector<EdgeType>>> BiC = new Vector<Pair<Vector<VertexType>, Vector<EdgeType>>>();

    int DFSn;
    private VertexType root;

    /**
     * The initialization, before searching for biconnected
     * components.
     * Here, DFSn is the number assigned to vertices in process
     * of dfs traversal, although in decreasing order.
     * High[v] denotes the highest vertex, reachable from v or
     * it's children in the dfs tree.
     *
     * @param g is the input graph.
     */
    private void init(BaseGraph<VertexType, EdgeType> g) {
        DFS_N = new Integer[g.getVerticesCount()];
        for (int i = 0; i < g.getVerticesCount(); i++)
            DFS_N[i] = 0;

        DFSn = g.getVerticesCount();
        High = new Integer[g.getVerticesCount()];
        Parent = new int[g.getVerticesCount()];
    }

    public void doAlgorithm() {
        GraphRequest<VertexType, EdgeType> gr = new GraphRequest<VertexType, EdgeType>();
        dispatchEvent(gr);
        BaseGraph<VertexType, EdgeType> g = gr.getGraph();
        this.Bicon(g);
    }

    /**
     * This class is used to put both vertices and edges in
     * a stack.
     */
    private class VE {

        VertexType v;
        EdgeType e;

        VE(VertexType v) {
            this.v = v;
            this.e = null;
        }

        VE(EdgeType e) {
            this.e = e;
            this.v = null;
        }

        VE() {
            this.v = null;
            this.e = null;
        }
    }

    int rootChilds = 0;
    int foundDecompositions;
    Stack<VE> S = new Stack<VE>(); //stack is initially empty

    /**
     * This method is in fact dfs, with some preworks and postworks
     * added to it.
     * It finds biconnected components of the input graph g, by
     * starting search from the node v.
     *
     * @param g
     * @param v
     */
    private void BC(BaseGraph<VertexType, EdgeType> g, VertexType v) {
        DFS_N[v.getId()] = DFSn;
        DFSn--;
        S.push(new VE(v));
        High[v.getId()] = DFS_N[v.getId()];

        Iterator<EdgeType> iet;
        iet = g.edgeIterator(v);

        EdgeType edge;
        VertexType w;
//        BaseGraph<VertexType,EdgeType> g1;

        while (iet.hasNext()) {
            edge = iet.next();
            w = edge.source == v ? edge.target : edge.source;
//            if (edge.getRGBColor()==0)
//                S.push(new VE(edge)); //each edge will be inserted twice (for both directions)

            if (Parent[v.getId()] != w.getId())
                if (DFS_N[w.getId()] == 0) { //w not visited before
                    Parent[w.getId()] = v.getId();
                    edge.setMark(true);
                    w.setMark(true);
                    EventUtils.algorithmStep(this, "");
                    BC(g, w);
                    w.setMark(false);
                    EventUtils.algorithmStep(this, "");
                    if (High[w.getId()] <= DFS_N[v.getId()]) { //v disconnects w from the rest of the graph
                        if (v == root) {
                            //the root is an articulation point if and only if it has more than one child in the DFS tree
                            rootChilds++;
                            if (rootChilds == 1)
                                continue;
                        }
                        foundDecompositions++;
                        //VE SObj = new VE();
                        //Vector<VertexType> ov = new Vector<VertexType>();
                        //Vector<EdgeType> oe = new Vector<EdgeType>();
                        v.setColor(2);
                        //the stact has problems, put it for later correction
//                        while (SObj.v != v) {
//                            //remove all edges and vertices from stack until v is reached, and mark the subgraph they form as a biconnected component
//                            SObj = S.pop();
//                            if (SObj.v != null) {
//                                SObj.v.setColor(foundDecompositions);
//                                ov.add(SObj.v);
//                            } else {
//                                EdgeType e = SObj.e;
////                                if (){
//                                    SObj.e.setColor(foundDecompositions);
//                                    oe.add(SObj.e);
////                                }
//                            }
//                        EventUtils.algorithmStep(this,600);
//                        }
//                        BiC.add(new Pair<Vector<VertexType>, Vector<EdgeType>>(ov, oe));
//                        S.add(new VE(v));//v is part of w's component and possibly others
                    }
                    High[v.getId()] = Math.max(High[v.getId()], High[w.getId()]);
//                    if (High[w.getId()] > High[v.getId()])
//                        High[v.getId()] = High[w.getId()];
                } else   //(v,w) is a back edge or a forward edge
                    High[v.getId()] = Math.max(High[v.getId()], DFS_N[w.getId()]);
        }
//        dispatchEvent(new VertexEdgeLabelEvent<VertexType, EdgeType>(DFS_N[v.getId()] + ":" + High[v.getId()], v));
    }


    /**
     * This is the main method, wich take the graph g as
     * the input, and returns the biconnected components
     * of g in a vector.
     *
     * @param g
     */
    public Vector<Pair<Vector<VertexType>, Vector<EdgeType>>> Bicon(BaseGraph<VertexType, EdgeType> g) {
//    public Vector<BaseGraph<VertexType,EdgeType>> Bicon(BaseGraph<VertexType,EdgeType> g){
        init(g);
        VertexType v = g.iterator().next();
        rootChilds = 0;
        root = v;
        foundDecompositions = 0;
        BC(g, v);
        return BiC;

    }
}