// GraphLab Project: http://graphlab.sharif.edu
// Copyright (C) 2008 Mathematical Science Department of Sharif University of Technology
// Distributed under the terms of the GNU General Public License (GPL): http://www.gnu.org/licenses/
package graphlab.platform.extension;

import graphlab.platform.StaticUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;

/**
 * @author azin azadi

 */
public class ExtensionClassLoader extends ClassLoader {
    public Map<String, byte[]> classesData = new HashMap<String, byte[]>();
    Map<String, Class> classes = new HashMap<String, Class>();
    Vector<File> unknownFiles = new Vector<File>();
    public static URLClassLoader classLoader;
    public static ClassLoader cl;

    public ExtensionClassLoader(String dirPath) {
        loadClasses(dirPath);
    }

    public Collection<Class> getLoadedClasses() {
        return classes.values();
    }

    public Vector<File> getUnknownFilesFound() {
        return unknownFiles;
    }


    Vector<URL> urls;

    private void loadClassFiles(String dir, String pack) {
        urls = new Vector<URL>();
        File file = new File(dir);
        if (!file.exists())
            return;
        File files[] = file.listFiles();
        for (File file1 : files)
            if (file1.isDirectory())
                loadClassFiles(dir + "/" + file1.getName(), pack + "." + file1.getName());
            else if (file1.getName().endsWith(".class"))
                try {
                    String name;
                    if (pack.length() > 0)
                        name = pack.substring(1) + ".";
                    else
                        name = "";
                    name = name + file1.getName().substring(0, file1.getName().length() - 6);
                    byte data[] = new byte[(int) file1.length()];
                    FileInputStream fis = new FileInputStream(file1);
                    fis.read(data);
                    classesData.put(name, data);
                    urls.add(file1.toURL());
                }
                catch (FileNotFoundException e) {
                    e.printStackTrace();
//                    ExceptionHandler.handleException(e);
                }
                catch (IOException e) {
                    e.printStackTrace();
//                    ExceptionHandler.handleException(e);
                }
            else {
                unknownFiles.add(file1);
            }

    }

    private void loadClasses(String dirPath) {
        loadClassFiles(dirPath, "");
//        defineClasses();
    }

    public Class findClass(String name) throws ClassNotFoundException {
        Class ret = null;
        if (!classesData.containsKey(name))
            ret = getParent().loadClass(name);
        if (ret == null && !classes.containsKey(name)) {
            byte data[] = (byte[]) classesData.get(name);
            Class c = defineClass(name, data, 0, data.length);
            classes.put(name, c);
        }
        ret = classes.get(name);
        if (ret == null)
            return classLoader.loadClass(name);
        else
            return ret;
    }

    public Collection getClassesImplementing(Class cl) {
        Collection col = new Vector();
        for (Map.Entry<String, Class> entry1 : classes.entrySet()) {
            Map.Entry entry = (Map.Entry) entry1;
            Class c = (Class) entry.getValue();
            if (StaticUtils.isImplementing(c, cl))
                col.add(c);
        }

        return col;
    }

}
