// GraphLab Project: http://graphlab.sharif.edu
// Copyright (C) 2008 Mathematical Science Department of Sharif University of Technology
// Distributed under the terms of the GNU General Public License (GPL): http://www.gnu.org/licenses/
package graphlab.plugins.main.core.actions.edge;

import graphlab.graph.GraphUtils;
import graphlab.graph.atributeset.GraphAttrSet;
import graphlab.graph.event.GraphEvent;
import graphlab.graph.event.VertexEvent;
import graphlab.graph.graph.*;
import graphlab.graph.old.AcceleratedRenderer;
import graphlab.graph.old.GStroke;
import graphlab.library.util.Pair;
import graphlab.platform.core.AbstractAction;
import graphlab.platform.core.BlackBoard;
import graphlab.plugins.commonplugin.undo.Undoable;
import graphlab.plugins.commonplugin.undo.UndoableActionOccuredData;
import graphlab.plugins.main.select.Select;
import graphlab.plugins.main.select.SelectPluginMethods;

import java.awt.*;
import java.awt.event.MouseEvent;

/**
 * @author azin azadi
 */

public class AddEdge extends AbstractAction implements PaintHandler, Undoable {
    //    private double lasty;
    //    private double lastx;
    private GraphPoint lastPos;
    private SelectPluginMethods selMethods;

    /**
     * constructor
     *
     * @param bb the blackboard of the action
     */
    public AddEdge(BlackBoard bb) {
        super(bb);
        selMethods = new SelectPluginMethods(bb);
        listen4Event(VertexEvent.EVENT_KEY);
//        listen4Event(GraphEvent.name);
//        listen4Event(VertexDropData.event);
//        listen4Event(VertexSelectData.name);
    }

    protected VertexModel v1, vc1;
    protected GraphModel g;
    protected AbstractGraphRenderer gv;
    protected boolean isClick = false;
    protected boolean isDrag = false;

    public void performAction(String eventName, Object value) {
// our old lovely Add Edge
//        if (name == VertexSelectData.name) {
//            VertexSelectData vsd = blackboard.getLog(VertexSelectData.name).getLast();
//            if (!isClick) {
//                vc1 = vsd.v;
//                v1=vc1;
//                startPainting();
//            } else {
//                Vertex v2=vsd.v;
//                Edge e = doJob(v2.g, v2, vc1);
//                addUndoData(e);
//                stopPainting();
//            }
//            isClick=true;
//            isDrag=false;
//        }
        VertexEvent ve = blackboard.getData(VertexEvent.EVENT_KEY);

        if ((ve.modifiers & MouseEvent.SHIFT_DOWN_MASK) == MouseEvent.SHIFT_DOWN_MASK)
            return;

        SubGraph sd = Select.getSelection(blackboard);
        g = blackboard.getData(GraphAttrSet.name);
        gv = blackboard.getData(AbstractGraphRenderer.EVENT_KEY);


        if (eventName == VertexEvent.EVENT_KEY) {
            if (ve.eventType == VertexEvent.DRAGGING_STARTED) {
                if (!sd.vertices.contains(ve.v)) {   //start if the vertex is not selected
//            VertexDragData vdrag = blackboard.get(VertexDragData.name);
                    v1 = ve.v;
                    startPainting();
//                    if (!selMethods.getSelectedVertices().contains(v1)) {
//                        //the vertex doesn't selected
//                    } else
//                        v1 = null;
                }
            }
            if (ve.eventType == VertexEvent.DROPPED) {
                if (v1 != null) {
                    VertexModel v2 = ve.v;
                    if (v2 != null && isDrag)   //!it was released on empty space
                        if (!v1.equals(v2)) {
                            EdgeModel e = doJob(g, v1, v2);
                            addUndoData(e, g);
                        }
//            unListenEvent(VertexMouseDraggingData.event);
                }
                stopPainting();
                v1 = null;
            }

            if (ve.eventType == VertexEvent.DRAGGING) {
                lastPos = ve.mousePos;
                GraphPoint absPosOnGraph = GraphUtils.createGraphPoint(g, x1, y1);
                absPosOnGraph.add(lastPos);
                if (g.getVerticesCount() < 300) {
                    Pair<VertexModel, Double> p = GraphControl.mindistv(g, absPosOnGraph);
                    if (p.first != null)
                        curVertexUnderMouse = GraphControl.isPointOnVertex(g, p.first, absPosOnGraph) ? p.first : null;
                }
                if (!(gv instanceof AcceleratedRenderer))
                    gv.repaint();

                isDrag = true;
                isClick = false;
            }

            if (ve.eventType == VertexEvent.DOUBLECLICKED) {
                VertexModel v = ve.v;
                if (v != null) {
                    EdgeModel e = doJob(g, v, v);
                    addUndoData(e, g);
                }
            }
            if (ve.eventType == VertexEvent.RELEASED) { //mouse released on a blank area of graph
                v1 = null;
                stopPainting();
            }
        } else if (eventName == GraphEvent.EVENT_KEY) {
//                if (ge.eventType == GraphEvent.) {
//                    stopPainting();
//                }
        }
    }

    protected void stopPainting() {
        gv.removePaintHandler(this);
//        g.view.repaint();
    }

    protected void addUndoData(EdgeModel e, GraphModel g) {
        UndoableActionOccuredData uaod = new UndoableActionOccuredData(this);
        uaod.properties.put("AddedEdge", e);
        uaod.properties.put("Graph", g);
        blackboard.setData(UndoableActionOccuredData.EVENT_KEY, uaod);
    }

    protected void startPainting() {
        GraphPoint location = v1.getLocation();
        Point viewPoint = GraphUtils.createViewPoint(g, location);
        cx = v1.getCenter().x / 2;
        x1 = viewPoint.x;
        cy = v1.getCenter().y / 2;
        y1 = viewPoint.y;
//        listen4Event(VertexMouseDraggingData.event);
        gv.addPostPaintHandler(this);
    }

    public static EdgeModel doJob(GraphModel g, VertexModel v1, VertexModel v2) {
        EdgeModel e = new EdgeModel(v1, v2);
        g.insertEdge(e);
        return e;
    }

    protected int x1;
    protected int y1;
    protected int cx;
    protected int cy;

    protected VertexModel curVertexUnderMouse = null;

    public void paint(Graphics g, Object component, Boolean drawExtras) {
        if (!drawExtras)
            return;
        Graphics2D gg = (Graphics2D) g;
        gg.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        Stroke stroke = gg.getStroke();
        gg.setStroke(GStroke.dashed.stroke);
        if (lastPos != null) {
            Point viewPoint = GraphUtils.createViewPoint(this.g, lastPos);
            gg.setColor(Color.DARK_GRAY);
            if (curVertexUnderMouse == null)
                gg.drawLine(x1, y1, x1 + viewPoint.x, y1 + viewPoint.y);
            else {
                gg.setStroke(GStroke.strong.stroke);
                GraphPoint loc = curVertexUnderMouse.getLocation();
                viewPoint = GraphUtils.createViewPoint(this.g, loc);
                gg.drawLine(x1, y1, viewPoint.x, viewPoint.y);
            }
        }
//            }
        gg.setStroke(stroke);
    }

    public void undo(UndoableActionOccuredData uaod) {
        EdgeModel e = (EdgeModel) uaod.properties.get("AddedEdge");
        GraphModel g = (GraphModel) uaod.properties.get("Graph");
        g.removeEdge(e);

    }

    public void redo(UndoableActionOccuredData uaod) {
        EdgeModel e = (EdgeModel) uaod.properties.get("AddedEdge");
        GraphModel g = (GraphModel) uaod.properties.get("Graph");
        g.insertEdge(e);
    }
}