// GraphLab Project: http://graphlab.sharif.edu
// Copyright (C) 2008 Mathematical Science Department of Sharif University of Technology
// Distributed under the terms of the GNU General Public License (GPL): http://www.gnu.org/licenses/
package graphlab.plugins.main.saveload.matrix;

import graphlab.graph.graph.EdgeModel;
import graphlab.graph.graph.GraphModel;
import graphlab.graph.graph.GraphPoint;
import graphlab.graph.graph.VertexModel;
import graphlab.library.BaseEdge;
import graphlab.library.BaseGraph;
import graphlab.library.BaseVertex;

import javax.swing.*;
import java.util.HashMap;
import java.util.Scanner;

/**
 * @author Azin Azadi
 */
public class Matrix {

    public static <vt extends BaseVertex, et extends BaseEdge<vt>>
    boolean[][] graph2Matrix(BaseGraph<vt, et> g) {
        int n = g.getVerticesCount();
        boolean[][] ret = new boolean[n][n];
        HashMap<Integer, vt> m = new HashMap<Integer, vt>();
        int i = 0;
        for (vt _ : g) {
            m.put(i++, _);
        }
        for (i = 0; i < n; i++)
            for (int j = 0; j < n; j++) {
                ret[i][j] = g.isEdge(m.get(i), m.get(j));
            }
        return ret;
    }

    public static void Matrix2Graph(boolean[][] mat, GraphModel g) {
        int n = mat.length;
        int nn = mat[1].length;
        if (n != nn) {
            JOptionPane.showMessageDialog(null, "not a valid matrix graph");
            return;
        }
        VertexModel[] vertices = new VertexModel[n];
        for (int i = 0; i < n; i++) {
            VertexModel v = new VertexModel();
            g.insertVertex(v);
            v.setLocation(new GraphPoint(Math.random() * 500, Math.random() * 500));
            vertices[i] = v;
        }
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (mat[i][j])
                    g.insertEdge(new EdgeModel(vertices[i], vertices[j]));
            }
        }
    }

    public static String Matrix2String(boolean[][] mat) {
        int a = mat.length;
        if (a == 0)
            return "";
        int b = mat[0].length;
        String ret = "";
        for (int i = 0; i < a; i++) {
            for (int j = 0; j < b; j++) {
                ret += (mat[i][j] ? "1" : "0");
                ret += " ";
            }
            ret += "\n";
        }
        return ret;
    }

    public static boolean[][] String2Matrix(String s) {
        int n = 0;
        for (int i = 0; s.charAt(i) != '\n'; i++) {
            char c = s.charAt(i);
            if (c == '1' || c == '0')
                n++;
        }
        System.out.println(n);
        boolean ret[][] = new boolean[n][n];
        Scanner sc = new Scanner(s);
        int i = 0;
        while (sc.hasNextLine()) {
            for (int j = 0; j < n; j++) {
                ret[i][j] = sc.nextInt() == 1;
            }
            i++;
            // ## \/ ?
            sc.nextLine();
        }
        return ret;
    }
}
