// GraphLab Project: http://graphlab.sharif.edu
// Copyright (C) 2008 Mathematical Science Department of Sharif University of Technology
// Distributed under the terms of the GNU General Public License (GPL): http://www.gnu.org/licenses/

package graphlab.plugins.reports.basicreports;

import Jama.EigenvalueDecomposition;
import graphlab.platform.lang.CommandAttitude;
import graphlab.plugins.main.GraphData;
import graphlab.plugins.reports.extension.GraphReportExtension;

import java.util.ArrayList;

/**
 * @author Mohammad Ali Rostami
 */

@CommandAttitude(name = "eig_values", abbreviation = "_evs")
public class AdjMatrixEigenValues implements GraphReportExtension {

    double round(double value, int decimalPlace) {
        double power_of_ten = 1;
        while (decimalPlace-- > 0)
            power_of_ten *= 10.0;
        return Math.round(value * power_of_ten)
                / power_of_ten;
    }

    public Object calculate(GraphData gd) {
        try {
            EigenvalueDecomposition ed = gd.getGraph().getAdjacencyMatrix().eig();
            double rv[] = ed.getRealEigenvalues();
            double iv[] = ed.getImagEigenvalues();

            ArrayList<String> eigvs = new ArrayList<String>();
            for (int i = 0; i < rv.length; i++)
                if (iv[i] != 0)
                    eigvs.add("" + round(rv[i], 3) + " + " + round(iv[i], 3) + "i");
                else
                    eigvs.add("" + round(rv[i], 3));
            return eigvs;
        } catch (Exception e) {
        }
        return "";
    }

    public String getName() {
        return "Eigen Values";
    }

    public String getDescription() {
        return "adjacency matrix eigen values";
    }
}
